#include "LSMTargz.h"
#include <zlib.h>
#pragma comment(lib,"zlib.lib")

Cgzipfile::Cgzipfile(void)
{
}

Cgzipfile::~Cgzipfile(void)
{
}

#define GZ_Debug printf

bool Cgzipfile::FromFile(const char *p_Filename)
{
	FILE *file=fopen_utf8(p_Filename,"rb");
	if (!file)
	{
		printf("gzip: Could not open %s",p_Filename);
		return false;
	}

	unsigned int FooterPosition=0;
	fseek(file,0,SEEK_END);
	FooterPosition=ftell(file);
	if (FooterPosition<8)
	{
		printf("gzip: Invalid gzip file %s",p_Filename);
		fclose(file);
		return false;
	}

	FooterPosition-=8;
	fseek(file,FooterPosition,SEEK_SET);

	gzFooter Footer;
	if (fread(&Footer,1,sizeof(Footer),file)!=sizeof(Footer))
	{
		printf("gzip: Failed to read from %s",p_Filename);
		fclose(file);
		return false;
	}
	fseek(file,0,SEEK_SET);

	gzHeader Header;
	if (fread(&Header,1,sizeof(Header),file)!=sizeof(Header))
	{
		printf("gzip: Failed to read from %s",p_Filename);
		fclose(file);
		return false;
	}
	if (Header.ID1!=0x1f || Header.ID2!=0x8b || Header.CM!=8)
	{
		printf("gzip: File not gzip format %s",p_Filename);
		fclose(file);
		return false;
	}

//	GZ_Debug("Header: FLG=0x%X MTIME=0x%X XFL=0x%X OS=0x%X",Header.FLG,Header.MTIME,Header.XFL,Header.OS);

	unsigned int Size=Data.Resize(Footer.ISIZE);
	if (Size!=Footer.ISIZE)
	{
		printf("gzip: Failed to initialize read buffer");
		fclose(file);
		return false;
	}

	if (Header.FEXTRA)
	{
		unsigned short Skip=0;
		if (fread(&Skip,1,2,file)!=2)
		{
			printf("gzip: Invalid gzip format %s",p_Filename);
			fclose(file);
			return false;
		}	
	}

	if (Header.FNAME)
	{
		char Byte=0;
		while(fread(&Byte,1,1,file) && Byte)
		{
		}
	}

	if (Header.FCOMMENT)
	{
		char Byte=0;
		while(fread(&Byte,1,1,file) && Byte)
		{
		}
	}

	if (Header.FHCRC)
	{
		fseek(file,2,SEEK_CUR);
	}

	unsigned int Position=ftell(file);
	unsigned int PayloadLength=FooterPosition-Position;

	char *Buffer=(char*)malloc(PayloadLength);
	if (fread(Buffer,1,PayloadLength,file)!=PayloadLength)
	{
		printf("gzip: Failed to read from %s",p_Filename);
		free(Buffer);
		fclose(file);
		return false;
	}

	fclose(file);

	z_stream z;
	memset(&z,0,sizeof(z_stream));
	unsigned long result=inflateInit2(&z,-MAX_WBITS);
	if (result!=Z_OK)
	{
		printf("gzip: Failed to initialize inflate algorithm",p_Filename);
		free(Buffer);
		return 0;
	}

	z.avail_in=PayloadLength;
	z.next_in=(Bytef*)Buffer;
	z.next_out=(Bytef*)Data.GetBuffer();
	z.avail_out=Footer.ISIZE;

	result=inflate(&z,Z_FINISH);
	if (result!=Z_OK && result!=Z_STREAM_END)
	{
		printf("gzip: inflate algorithm failed",p_Filename);
		inflateEnd(&z);
		free(Buffer);
		return 0;
	}
	inflateEnd(&z);
	free(Buffer);

	Data.SetUsed(z.total_out);
	if (z.total_out!=Footer.ISIZE)
	{
		printf("gzip: Output size mismatch in %s",p_Filename);
		return false;
	}
	return true;
}

