#include "LSMTargz.h"

int CMD_tar(int argc, char *argv[])
{
	if (argc<2)
	{
		printf("Syntax: %s [OPTION...] [FILE]...",argv[0]);
		printf("Currently supported options are xzvf, and must be prepended with -");
		printf("Example: %s -xzf foo.tar.gz",argv[0]);
		return 0;
	}
	bool bVerbose=false;
	bool bGZip=false;
	char Mode=0;
	char *archive=0;

	// read options
	int arg=1;
	for (arg ; arg < argc ; arg++)
	{
		if (argv[arg][0]!='-')
			break;

		char *opt=&argv[arg][1];
		while(char c=*opt)
		{
			switch(c)
			{
			case 'v':
				bVerbose=true;
				break;
			case 'z':
				bGZip=true;
				break;
			case 'f':
				if (arg+1<argc)
				{
					arg++;
					archive=argv[arg];
				}
				else
				{
					printf("%s: option requires an argument -- f",argv[0]);
					return -1;
				}
				break;
			case 'x':
			case 'A':
			case 'c':
			case 't':
			case 'u':
				Mode=c;
				break;
			}
			opt++;
		}
		
	}

	if (!archive)
	{
		printf("No archive specified");
		return 0;
	}

	char FullFilename[512];
	pLSInterface->SetRoot(archive,FullFilename);

	switch(Mode)
	{
	case 'x':
		{// Extract
			Ctar Tar;
			if (bGZip)
			{
				Cgzipfile GZip;
				if (!GZip.FromFile(FullFilename))
				{
					printf("Failed to read %s",FullFilename);
					return -1;
				}
				if (!Tar.FromBuffer(GZip.Data.GetBuffer(),GZip.Data.GetUsed()))
				{
					printf("Invalid tar in %s",FullFilename);
					return -1;
				}
			}
			else
			{
				if (!Tar.FromFile(FullFilename))
				{
					printf("Failed to read %s",FullFilename);
					return -1;
				}
			}
			// tar is valid, extract
			for (map<utf8stringnocase,Ctarfile*>::iterator i=Tar.Files.begin() ; i!=Tar.Files.end() ; i++)
			{
				if (bVerbose)
				{
					printf("%s",i->first.c_str());
				}
				char TempFilename[512];
				strcpy(TempFilename,i->first.c_str());
				if (char *pSlash=strchr(TempFilename,'/'))
				{
					do
					{
						*pSlash='\\';
					}
					while (pSlash=strchr(&pSlash[1],'/'));
				}
				pLSInterface->SetRoot(TempFilename,FullFilename);
				if (FILE *file=fopen_utf8(FullFilename,"wb"))
				{
					fwrite(i->second->Data.GetBuffer(),1,i->second->Data.GetUsed(),file);
					fclose(file);
				}
				else
				{
					printf("Failed to open '%s' for writing",FullFilename);
				}
			}
		}
		break;
	case 0:
		printf("%s: You must specify one of the `-xActu' options",argv[0]);
		break;
	default:
		printf("%s: Mode '%c' not supported",argv[0],Mode);
		break;
	}
	return 0;
}

int CMD_gzip(int argc, char *argv[])
{
	return 0;
}
