#include "LSMTargz.h"

Ctar::Ctar(void):LSIteratable("tar")
{
}

Ctar::~Ctar(void)
{
	Clear();
}

void Ctar::Clear()
{
	for (map<utf8stringnocase,Ctarfile*>::iterator i=Files.begin(); i!=Files.end() ; i++)
	{
		delete i->second;
	}
	Files.clear();
}

bool Ctar::FromBuffer(const void *p_vBuffer, unsigned int p_Size)
{
	const char *p_Buffer=(const char *)p_vBuffer;
	while(p_Size>=512)
	{
		_USTARHeader *pHeader=(_USTARHeader *)p_Buffer;

		if (!Ctarfile::VerifyChecksum(*pHeader))
		{
			printf("tar: invalid checksum at block %d",((unsigned int)pHeader-(unsigned int)p_vBuffer)/512);
			return false;
		}

		unsigned int FileSize=0;
		sscanf(pHeader->FileSize,"%o",&FileSize);
		p_Buffer+=512;
		p_Size-=512;

		unsigned int Padding=FileSize%512;
		if (Padding)
			Padding=512-Padding;

		if (p_Size<FileSize+Padding)
		{
			printf("tar: invalid size (%d<=%d+%d)",p_Size,FileSize,Padding);
			return false;
		}
		
		char ActualFilename[256];
		sprintf(ActualFilename,"%s%s",pHeader->FilenamePrefix,pHeader->Filename);

		if (ActualFilename[0]) // make sure this block isn't purely padding
		{
			if (FindFile(ActualFilename))
			{
				RemoveFile(ActualFilename);
//				printf("tar: file already in archive");
//				return false;
			}
			Ctarfile *pFile=new Ctarfile(*pHeader,p_Buffer,FileSize);
			Files[ActualFilename]=pFile;
//			printf("Tarred file: %s (size=%d)",ActualFilename,FileSize);
		}
		
		p_Buffer+=FileSize+Padding;
		p_Size-=FileSize+Padding;
	}

	return true;
}
bool Ctar::FromFile(const char *p_FileName)
{
	FILE *file=fopen_utf8(p_FileName,"rb");
	if (!file)
		return false;

	fseek(file,0,SEEK_END);
	unsigned long FileSize=ftell(file);
	if (FileSize%512)
	{
		fclose(file); // not a valid .tar
		return false;
	}
	fseek(file,0,SEEK_SET);
	char *Buffer=(char*)malloc(FileSize);
	bool Ret=false;
	if (fread(Buffer,1,FileSize,file)==FileSize)
	{
		Ret=FromBuffer(Buffer,FileSize);
	}
	fclose(file);
	return Ret;
}

bool Ctar::ToFile(const char *p_FileName)
{
	FILE *file=fopen_utf8(p_FileName,"wb");
	if (!file)
		return false;
	
	char Block[512];
	static char PaddingBlock[512];
	ZeroMemory(PaddingBlock,sizeof(PaddingBlock));
	for (map<utf8stringnocase,Ctarfile*>::iterator i=Files.begin() ; i!=Files.end() ; i++)
	{
		Ctarfile *pFile=i->second;
		// output header
		ZeroMemory(Block,sizeof(Block));
		_USTARHeader *pHeader=(_USTARHeader *)&Block;

		// the data and/or header may have been updated in memory. update file size and header checksum.
		unsigned int FileSize=pFile->Data.GetUsed();
		sprintf(pFile->Header.FileSize,"%011o",FileSize);
		pFile->UpdateChecksum();

		*pHeader=pFile->Header;
		fwrite(Block,1,512,file);
	
		// output file
		fwrite(pFile->Data.GetBuffer(),1,FileSize,file);

		// pad end of file
		unsigned int Padding=FileSize%512;
		if (Padding)
			Padding=512-Padding;
		fwrite(PaddingBlock,1,Padding,file);
	}
	// end archive with at least two blocks of padding
	fwrite(PaddingBlock,1,512,file);
	fwrite(PaddingBlock,1,512,file);

	fclose(file);
	return true;
}

bool Ctar::AddFileFromBuffer(const char *p_FileName,const void *buf,unsigned int buflen)
{
	// TODO
	return false;
}

bool Ctar::AddFile(const char *p_FileName)
{
	// TODO
	return false;
}
bool Ctar::RemoveFile(const char *p_FileName)
{
	Ctarfile *pFile=FindFile(p_FileName);
	if (!pFile)
		return false;
	delete pFile;
	Files.erase(p_FileName);
	return true;
}

LSIterator *Ctar::NewIterator(void *PassAlong)
{
	// TODO
	return 0;
}
