#pragma once


/*
0 	100 	File name
100 	8 	File mode
108 	8 	Owner user ID
116 	8 	Group user ID
124 	12 	File size in bytes
136 	12 	Last modification time
148 	8 	Check sum for header block
156 	1 	Type flag
157 	100 	Name of linked file
257 	6 	USTAR indicator
263 	2 	USTAR version
265 	32 	Owner user name
297 	32 	Owner group name
329 	8 	Device major number
337 	8 	Device minor number
345 	155 	Filename prefix
/**/

#pragma pack(push)
#pragma pack(1)
struct _USTARHeader
{
/*0  */	char Filename[100];
/*100*/	char FileMode[8];
/*108*/	char OwnwerUID[8];
/*116*/	char GroupUID[8];
/*124*/	char FileSize[12];
/*136*/	char FileTime[12];
/*148*/	char Checksum[8];
/*156*/	char TypeFlag;
/*157*/	char LinkFilename[100];
/*257*/ char USTAR[8]; // "ustar  "
/*265*/	char OwnerName[32];
/*297*/ char OwnerGroup[32];
/*329*/ char DeviceMajor[8];
/*337*/ char DeviceMinor[8];
/*345*/ char FilenamePrefix[155];
};
#pragma pack(pop)

class Ctarfile
{
public:
	Ctarfile(_USTARHeader &p_Header,const void *buf, unsigned int buflen):Data((void*)buf,buflen)
	{
		Header=p_Header;
	}
	virtual ~Ctarfile()
	{

	}
	_USTARHeader Header;

	void UpdateChecksum()
	{
		memset(Header.Checksum,32,8);
		// calculate sum
		unsigned int Sum=0;
		unsigned char *Bytes=(unsigned char *)&Header;
		for (unsigned int i = 0 ; i < 148 ; i++)
		{
			Sum+=Bytes[i];
		}
		for (unsigned int i = 156 ; i < sizeof(Header) ; i++)
		{
			Sum+=Bytes[i];
		}
		if (Sum)
			Sum+=8*32;
		// output sum
		sprintf(Header.Checksum,"%06o",Sum);
	}

	static bool VerifyChecksum(_USTARHeader &Header)
	{
		// calculate sum
		unsigned int Sum=0;
		unsigned char *Bytes=(unsigned char *)&Header;
		for (unsigned int i = 0 ; i < 148 ; i++)
		{
			Sum+=Bytes[i];
		}
		for (unsigned int i = 156 ; i < sizeof(Header) ; i++)
		{
			Sum+=Bytes[i];
		}
		if (!Sum)
			return true;
		Sum+=8*32;
		// output sum
		//sprintf(Header.Checksum,"%06o",Sum);

		unsigned int StoredSum=0;
		sscanf(Header.Checksum,"%o",&StoredSum);
		return StoredSum==Sum;
	}

	LSBinary Data;
};

class Ctar : public LSIteratable
{
public:
	Ctar(void);
	virtual ~Ctar(void);

	void Clear();

	bool FromBuffer(const void *p_Buffer, unsigned int p_Size);
	bool FromFile(const char *p_FileName);
	bool ToFile(const char *p_FileName);

	bool AddFile(const char *p_FileName);
	bool AddFileFromBuffer(const char *p_FileName,const void *buf,unsigned int buflen);

	bool RemoveFile(const char *p_FileName);

	Ctarfile *FindFile(const char *p_FileName)
	{
		map<utf8stringnocase,Ctarfile*>::iterator i=Files.find(p_FileName);
		if (i==Files.end())
			return 0;
		return i->second;
	}

	virtual LSIterator *NewIterator(void *PassAlong);

//protected:
	map<utf8stringnocase,Ctarfile*> Files;
	CSemaphore S;
//	char *Buffer;
//	unsigned int Size;
};
